@@ -46,6 +46,7 @@ from neutron_lib.api.definitions import floating_ip_port_forwarding
 from neutron_lib.api.definitions import floatingip_autodelete_internal
 from neutron_lib.api.definitions import floatingip_pools
 from neutron_lib.api.definitions import flowclassifier
+from neutron_lib.api.definitions import interconnection
 from neutron_lib.api.definitions import ip_allocation
 from neutron_lib.api.definitions import ip_substring_port_filtering
 from neutron_lib.api.definitions import l2_adjacency
@@ -81,6 +82,7 @@ from neutron_lib.api.definitions import qos_default
 from neutron_lib.api.definitions import qos_gateway_ip
 from neutron_lib.api.definitions import qos_rule_type_details
 from neutron_lib.api.definitions import qos_rules_alias
+from neutron_lib.api.definitions import rbac_security_groups
 from neutron_lib.api.definitions import revisionifmatch
 from neutron_lib.api.definitions import router_availability_zone
 from neutron_lib.api.definitions import router_interface_fip
@@ -97,7 +99,9 @@ from neutron_lib.api.definitions import subnet
 from neutron_lib.api.definitions import subnet_onboard
 from neutron_lib.api.definitions import subnet_segmentid_enforce
 from neutron_lib.api.definitions import subnet_segmentid_writable
+from neutron_lib.api.definitions import subnet_service_types
 from neutron_lib.api.definitions import subnetpool
+from neutron_lib.api.definitions import subnetpool_prefix_ops
 from neutron_lib.api.definitions import trunk
 from neutron_lib.api.definitions import trunk_details
 from neutron_lib.api.definitions import uplink_status_propagation
@@ -143,6 +147,7 @@ _ALL_API_DEFINITIONS = {
     floating_ip_port_forwarding,
     floatingip_autodelete_internal,
     floatingip_pools,
+    interconnection,
     ip_allocation,
     ip_substring_port_filtering,
     l2_adjacency,
@@ -179,6 +184,7 @@ _ALL_API_DEFINITIONS = {
     qos_gateway_ip,
     qos_rule_type_details,
     qos_rules_alias,
+    rbac_security_groups,
     revisionifmatch,
     router_availability_zone,
     router_interface_fip,
@@ -195,7 +201,9 @@ _ALL_API_DEFINITIONS = {
     subnet_onboard,
     subnet_segmentid_enforce,
     subnet_segmentid_writable,
+    subnet_service_types,
     subnetpool,
+    subnetpool_prefix_ops,
     trunk,
     trunk_details,
     uplink_status_propagation,
