@@ -42,6 +42,7 @@ KNOWN_ATTRIBUTES = (
     'prefixlen',
     'project_id',
     'qos_policy_id',
+    'service_types',
     constants.SHARED,
     'status',
     'subnets',
@@ -123,6 +124,7 @@ KNOWN_EXTENSIONS = (
     'qos-rules-alias',
     'quotas',
     'rbac-policies',
+    'rbac-security-groups',
     'router',
     'router_availability_zone',
     'security-group',
@@ -134,8 +136,10 @@ KNOWN_EXTENSIONS = (
     'standard-attr-revisions',
     'standard-attr-segment',
     'standard-attr-timestamp',
+    'subnet',
     'subnet_allocation',
     'subnet_onboard',
+    'subnetpool-prefix-ops',
     'subnet-segmentid-enforce',
     'subnet-segmentid-writable',
     'tag',
@@ -160,6 +164,9 @@ KNOWN_EXTENSIONS = (
     # http://git.openstack.org/cgit/openstack/networking-sfc:
     'flow_classifier',
     'sfc',
+
+    # http://git.openstack.org/cgit/openstack/neutron-interconnection:
+    'interconnection',
 )
 
 KNOWN_KEYWORDS = (
