@@ -78,6 +78,7 @@ ROUTER_INTERFACE_OWNERS_SNAT = (DEVICE_OWNER_ROUTER_INTF,
 DEVICE_ID_RESERVED_DHCP_PORT = 'reserved_dhcp_port'
 
 FLOATINGIP_KEY = '_floatingips'
+PORT_FORWARDING_FLOATINGIP_KEY = '_pf_floatingips'
 INTERFACE_KEY = '_interfaces'
 HA_INTERFACE_KEY = '_ha_interface'
 
@@ -134,8 +135,10 @@ PROTO_NAME_DCCP = 'dccp'
 PROTO_NAME_EGP = 'egp'
 PROTO_NAME_ESP = 'esp'
 PROTO_NAME_GRE = 'gre'
+PROTO_NAME_HOPOPT = 'hopopt'
 PROTO_NAME_ICMP = 'icmp'
 PROTO_NAME_IGMP = 'igmp'
+PROTO_NAME_IP = 'ip'
 PROTO_NAME_IPIP = 'ipip'
 PROTO_NAME_IPV6_ENCAP = 'ipv6-encap'
 PROTO_NAME_IPV6_FRAG = 'ipv6-frag'
@@ -160,8 +163,10 @@ PROTO_NUM_DCCP = 33
 PROTO_NUM_EGP = 8
 PROTO_NUM_ESP = 50
 PROTO_NUM_GRE = 47
+PROTO_NUM_HOPOPT = 0
 PROTO_NUM_ICMP = 1
 PROTO_NUM_IGMP = 2
+PROTO_NUM_IP = 0
 PROTO_NUM_IPIP = 4
 PROTO_NUM_IPV6_ENCAP = 41
 PROTO_NUM_IPV6_FRAG = 44
@@ -183,8 +188,10 @@ IP_PROTOCOL_MAP = {PROTO_NAME_AH: PROTO_NUM_AH,
                    PROTO_NAME_EGP: PROTO_NUM_EGP,
                    PROTO_NAME_ESP: PROTO_NUM_ESP,
                    PROTO_NAME_GRE: PROTO_NUM_GRE,
+                   PROTO_NAME_HOPOPT: PROTO_NUM_HOPOPT,
                    PROTO_NAME_ICMP: PROTO_NUM_ICMP,
                    PROTO_NAME_IGMP: PROTO_NUM_IGMP,
+                   PROTO_NAME_IP: PROTO_NUM_IP,
                    PROTO_NAME_IPIP: PROTO_NUM_IPIP,
                    PROTO_NAME_IPV6_ENCAP: PROTO_NUM_IPV6_ENCAP,
                    PROTO_NAME_IPV6_FRAG: PROTO_NUM_IPV6_FRAG,
