@@ -15,10 +15,14 @@
 
 import functools
 import re
+import uuid
+
+import requests
 
 from keystoneauth1 import exceptions as ks_exc
 from keystoneauth1 import loading as keystone
 from oslo_log import log as logging
+from oslo_serialization import jsonutils
 from oslo_utils import versionutils
 from six.moves.urllib.parse import urlencode
 
@@ -40,8 +44,6 @@ def _check_placement_api_available(f):
 
     :param f: Function to execute.
     :returns: The returned value of the function f.
-    :raises PlacementEndpointNotFound: If the placement API endpoint configured
-                                       is not found.
     """
     @functools.wraps(f)
     def wrapper(self, *a, **k):
@@ -49,18 +51,17 @@ def _check_placement_api_available(f):
             if not self._client:
                 self._client = self._create_client()
             return f(self, *a, **k)
-        except ks_exc.EndpointNotFound:
-            LOG.warning('Please enable the placement service.')
-        except ks_exc.MissingAuthPlugin:
-            LOG.warning('No authentication information found for placement '
-                        'API. Please enable the placement service.')
-        except ks_exc.Unauthorized:
-            LOG.warning('Placement service credentials do not work. '
-                        'Please enable the placement service.')
-        except ks_exc.DiscoveryFailure:
-            LOG.warning('Discovering suitable URL for placement API failed.')
-        except ks_exc.ConnectFailure:
-            LOG.warning('Placement API service is not responding.')
+        except ks_exc.http.HttpError as exc:
+            if 400 <= exc.http_status <= 499:
+                # NOTE(bence romsics): Placement has inconsistently formatted
+                # error messages. Some error response bodies are JSON
+                # formatted, seemingly machine readible objects. While others
+                # are free format text. We have to keep the whole thing
+                # to avoid losing information.
+                raise n_exc.PlacementClientError(
+                    msg=exc.response.text.replace('\n', ' '))
+            else:
+                raise
     return wrapper
 
 
@@ -70,6 +71,64 @@ def _get_version(openstack_api_version):
     return versionutils.convert_version_to_tuple(match.group('api_version'))
 
 
+class UUIDEncoder(jsonutils.JSONEncoder):
+    def default(self, o):
+        if isinstance(o, uuid.UUID):
+            return str(o)
+        return super(UUIDEncoder, self).default(o)
+
+
+class NoAuthClient(object):
+    """Placement NoAuthClient for fullstack testing"""
+
+    def __init__(self, url):
+        self.url = url
+        # TODO(lajoskatona): use perhaps http_connect_timeout from
+        # keystone_authtoken group
+        self.timeout = 5
+
+    def request(self, url, method, body=None, headers=None, **kwargs):
+        headers = headers or {}
+        headers.setdefault('Accept', 'application/json')
+
+        # Note(lajoskatona): placement report plugin fills uuid fields with
+        # UUID objects, and that is good for keystone, but requests goes mad
+        # with that if we use json=body as it can't serialize UUID.
+        # To make things worse if we give a serialized json, it will do the
+        # jsonification again, so better to create the json here and give it
+        # to requests with the data parameter.
+        body = jsonutils.dumps(body, cls=UUIDEncoder)
+        try:
+            resp = requests.request(
+                method,
+                url,
+                data=body,
+                headers=headers,
+                verify=False,
+                timeout=self.timeout,
+                **kwargs)
+            return resp
+        # Note(lajoskatona): requests raise ConnectionError, but
+        # PlacementReportPlugin expects keystonauth1 HttpError.
+        except requests.ConnectionError:
+            raise ks_exc.HttpError
+
+    def get(self, url, endpoint_filter, **kwargs):
+        return self.request('%s%s' % (self.url, url), 'GET', **kwargs)
+
+    def post(self, url, json, endpoint_filter, **kwargs):
+        return self.request('%s%s' % (self.url, url), 'POST', body=json,
+                            **kwargs)
+
+    def put(self, url, json, endpoint_filter, **kwargs):
+        resp = self.request('%s%s' % (self.url, url), 'PUT', body=json,
+                            **kwargs)
+        return resp
+
+    def delete(self, url, endpoint_filter, **kwargs):
+        return self.request('%s%s' % (self.url, url), 'DELETE', **kwargs)
+
+
 class PlacementAPIClient(object):
     """Client class for placement ReST API."""
 
@@ -90,11 +149,18 @@ class PlacementAPIClient(object):
         # clean slate.
         self._resource_providers = {}
         self._provider_aggregate_map = {}
-        auth_plugin = keystone.load_auth_from_conf_options(
-            self._conf, 'placement')
-        return keystone.load_session_from_conf_options(
-            self._conf, 'placement', auth=auth_plugin,
-            additional_headers={'accept': 'application/json'})
+        # TODO(lajoskatona): perhaps not the best to override config options,
+        # actually the abused keystoneauth1 options are:
+        # auth_type (used for deciding for NoAuthClient) and auth_section
+        # (used for communicating the url for the NoAuthClient)
+        if self._conf.placement.auth_type == 'noauth':
+            return NoAuthClient(self._conf.placement.auth_section)
+        else:
+            auth_plugin = keystone.load_auth_from_conf_options(
+                self._conf, 'placement')
+            return keystone.load_session_from_conf_options(
+                self._conf, 'placement', auth=auth_plugin,
+                additional_headers={'accept': 'application/json'})
 
     def _extend_header_with_api_version(self, **kwargs):
         headers = kwargs.get('headers', {})
@@ -146,6 +212,10 @@ class PlacementAPIClient(object):
                                   the uuid (required),
                                   the name (required) and
                                   the parent_provider_uuid (optional).
+        :raises PlacementResourceProviderNotFound: No such resource provider.
+        :raises PlacementResourceProviderNameNotUnique: Conflict with another
+                                                        resource provider with
+                                                        the same name.
         :returns: The updated resource provider.
         """
         url = '/resource_providers/%s' % resource_provider['uuid']
@@ -193,10 +263,8 @@ class PlacementAPIClient(object):
         """Get resource provider by UUID.
 
         :param resource_provider_uuid: UUID of the resource provider.
-        :raises PlacementResourceProviderNotFound: If the resource provider is
-                                                   not present.
-        :returns: The Resource Provider matching the UUID.
         :raises PlacementResourceProviderNotFound: For failure to find resource
+        :returns: The Resource Provider matching the UUID.
         """
         url = '/resource_providers/%s' % resource_provider_uuid
         try:
@@ -222,9 +290,9 @@ class PlacementAPIClient(object):
                         resource provider with UUID == tree_uuid.
                         NOTE: placement 1.14 needed.
         :param uuid: UUID of the resource provider.
-        :returns: A list of Resource Provider matching the filters.
         :raises PlacementAPIVersionIncorrect: If placement API target version
                                               is too low
+        :returns: A list of Resource Provider matching the filters.
         """
         url = '/resource_providers'
         filters = {}
@@ -256,7 +324,7 @@ class PlacementAPIClient(object):
     def update_resource_provider_inventories(
             self, resource_provider_uuid, inventories,
             resource_provider_generation=None):
-        """Update resource provider inventories.
+        """Replaces the set of inventory records for a resource provider.
 
         :param resource_provider_uuid: UUID of the resource provider.
         :param inventories: The inventories. A dict in the format (see:
@@ -278,6 +346,7 @@ class PlacementAPIClient(object):
                                                              provider does not
                                                              match with the
                                                              server side.
+        :returns: The updated set of inventory records.
         """
         if resource_provider_generation is None:
             resource_provider_generation = self.get_resource_provider(
@@ -304,6 +373,7 @@ class PlacementAPIClient(object):
         :param resource_provider_uuid: UUID of the resource provider.
         :raises PlacementResourceProviderNotFound: If the resource provider
                                                    is not found.
+        :returns: None.
         """
         url = '/resource_providers/%s/inventories' % (
             resource_provider_uuid)
@@ -323,6 +393,10 @@ class PlacementAPIClient(object):
 
         :param resource_provider_uuid: UUID of the resource provider.
         :param resource_class: The name of the resource class
+        :raises PlacementResourceProviderNotFound: If the resource provider
+                                                   is not found.
+        :raises PlacementInventoryNotFound: No inventory of class.
+        :returns: None.
         """
         url = '/resource_providers/%s/inventories/%s' % (
             resource_provider_uuid, resource_class)
@@ -346,8 +420,11 @@ class PlacementAPIClient(object):
         :param resource_provider_uuid: UUID of the resource provider.
         :param resource_class: Resource class name of the inventory to be
                                returned.
+        :raises PlacementResourceProviderNotFound: If the resource provider is
+                                                   not found.
         :raises PlacementInventoryNotFound: For failure to find inventory
                                             for a resource provider.
+        :returns: The inventory of the resource class as a dict.
         """
         url = '/resource_providers/%s/inventories/%s' % (
             resource_provider_uuid, resource_class)
@@ -380,6 +457,7 @@ class PlacementAPIClient(object):
         :raises PlacementInventoryUpdateConflict: If the resource provider
                                                   generation does not match
                                                   with the server side.
+        :returns: The updated inventory of the resource class as a dict.
         """
         if resource_provider_generation is None:
             resource_provider_generation = self.get_resource_provider(
@@ -416,6 +494,8 @@ class PlacementAPIClient(object):
         :param resource_provider_uuid: UUID of the resource provider.
         :raises PlacementAggregateNotFound: For failure to the aggregates of
                                             a resource provider.
+        :returns: The list of aggregates together with the resource provider
+                  generation.
         """
         url = '/resource_providers/%s/aggregates' % resource_provider_uuid
         try:
@@ -436,6 +516,7 @@ class PlacementAPIClient(object):
 
         :param name: name of the trait to check.
         :raises PlacementTraitNotFound: If the trait name not found.
+        :returns: Evaluates to True if the trait exists.
         """
         url = '/traits/%s' % name
         try:
@@ -448,6 +529,7 @@ class PlacementAPIClient(object):
         """Insert a single custom trait.
 
         :param name: name of the trait to create.
+        :returns: The Response object so you may access response headers.
         """
         url = '/traits/%s' % (name)
         return self._put(url, None)
@@ -457,6 +539,8 @@ class PlacementAPIClient(object):
         """Delete the specified trait.
 
         :param name: the name of the trait to be deleted.
+        :raises PlacementTraitNotFound: If the trait did not exist.
+        :returns: None.
         """
         url = '/traits/%s' % (name)
         try:
@@ -468,7 +552,7 @@ class PlacementAPIClient(object):
     def update_resource_provider_traits(
             self, resource_provider_uuid, traits,
             resource_provider_generation=None):
-        """Update resource provider traits
+        """Replace all associated traits of a resource provider.
 
         :param resource_provider_uuid: UUID of the resource provider for which
                                        to set the traits
@@ -479,6 +563,8 @@ class PlacementAPIClient(object):
                                                    is not found.
         :raises PlacementTraitNotFound: If any of the specified traits are not
                                         valid.
+        :returns: The new traits of the resource provider together with the
+                  resource provider generation.
         """
         if resource_provider_generation is None:
             resource_provider_generation = self.get_resource_provider(
@@ -504,6 +590,8 @@ class PlacementAPIClient(object):
                                        the traits will be listed
         :raises PlacementResourceProviderNotFound: If the resource provider
                                                    is not found.
+        :returns: The associated traits of the resource provider together
+                  with the resource provider generation.
         """
         url = '/resource_providers/%s/traits' % (resource_provider_uuid)
         try:
@@ -518,6 +606,9 @@ class PlacementAPIClient(object):
 
         :param resource_provider_uuid: The UUID of the resource provider for
                                        which to delete all the traits.
+        :raises PlacementResourceProviderNotFound: If the resource provider
+                                                   is not found.
+        :returns: None.
         """
         url = '/resource_providers/%s/traits' % (resource_provider_uuid)
         try:
@@ -537,6 +628,9 @@ class PlacementAPIClient(object):
         """Show resource class.
 
         :param name: The name of the resource class to show
+        :raises PlacementResourceClassNotFound: If the resource class
+                                                is not found.
+        :returns: The name of resource class and its set of links.
         """
         url = '/resource_classes/%s' % (name)
         try:
@@ -549,6 +643,7 @@ class PlacementAPIClient(object):
         """Create a custom resource class
 
         :param name: the name of the resource class
+        :returns: None.
         """
         url = '/resource_classes'
         body = {'name': name}
@@ -559,6 +654,7 @@ class PlacementAPIClient(object):
         """Create or validate the existence of the resource custom class.
 
         :param name: the name of the resource class to be updated or validated
+        :returns: None.
         """
         url = '/resource_classes/%s' % name
         self._put(url, None)
@@ -568,6 +664,9 @@ class PlacementAPIClient(object):
         """Delete a custom resource class.
 
         :param name: The name of the resource class to be deleted.
+        :raises PlacementResourceClassNotFound: If the resource class
+                                                is not found.
+        :returns: None.
         """
         url = '/resource_classes/%s' % (name)
         try:
