@@ -10,8 +10,9 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
+import re
 
+import mock
 from oslo_config import cfg
 from oslo_db import options
 from oslotest import base
@@ -60,10 +61,12 @@ class SqlFixtureTestCase(base.BaseTestCase):
         options.set_defaults(
             cfg.CONF,
             connection='sqlite://')
-        self.useFixture(fixture.SqlFixture())
+        self.fixture = fixture._SqlFixture()
+        self.useFixture(self.fixture)
 
     def test_fixture(self):
         self.assertIsNotNone(model_base.BASEV2.metadata.sorted_tables)
+        self.assertIsNotNone(self.fixture.engine)
 
 
 class APIDefinitionFixtureTestCase(base.BaseTestCase):
@@ -164,17 +167,33 @@ class DBResourceExtendFixtureTestCase(base.BaseTestCase):
             'a': 'A',
             'b': 'B'
         }
-        orig_methods = resource_extend._DECORATED_EXTEND_METHODS
+        orig_methods = resource_extend._resource_extend_functions
         self.assertNotEqual(fake_methods, orig_methods)
 
         db_fixture = fixture.DBResourceExtendFixture(
-            extended_methods=fake_methods)
+            extended_functions=fake_methods)
         db_fixture.setUp()
 
         resource_extend.register_funcs('C', (lambda x: x,))
         self.assertNotEqual(
-            orig_methods, resource_extend._DECORATED_EXTEND_METHODS)
+            orig_methods, resource_extend._resource_extend_functions)
 
         db_fixture.cleanUp()
         self.assertEqual(
-            orig_methods, resource_extend._DECORATED_EXTEND_METHODS)
+            orig_methods, resource_extend._resource_extend_functions)
+
+
+class WarningsFixture(base.BaseTestCase):
+
+    @mock.patch.object(fixture.warnings, 'filterwarnings')
+    def test_fixture_regex(self, mock_filterwarnings):
+        module_re = ['^neutron\\.']
+        warn_fixture = fixture.WarningsFixture(module_re=module_re)
+        warn_fixture.setUp()
+        call_re = mock_filterwarnings.mock_calls[0][2]['module']
+        self.assertEqual('^neutron_lib\\.|^neutron\\.', call_re)
+        self.assertIsNotNone(re.compile(call_re))
+        self.assertIsNotNone(re.search(call_re, 'neutron.db.blah'))
+        self.assertIsNotNone(re.search(call_re, 'neutron_lib.db.blah'))
+        self.assertIsNone(re.search(
+            call_re, 'neutron_dynamic_routing.db.blah'))
